#!/usr/bin/env bun
import glob from 'fast-glob';
import fs from 'fs';
import { builtinModules } from 'module';
import { uniq } from 'lodash';
import { dependencies, peerDependencies } from '../dist/package.json';

const deps = Object.keys({ ...dependencies, ...peerDependencies }).concat('eslint');
const builtIn = new Set(builtinModules.flatMap(module => [module, `node:${module}`]));

function findRequires(text: string) {
  const list = Array.from(text.matchAll(/require\(["']([^"']+)["']\)/g))
    .map(m => m[1])
    .filter(
      module =>
        !(
          builtIn.has(module) ||
          module.startsWith('eslint/') ||
          module.startsWith('typescript/')
        ),
    );
  return uniq(list);
}

const moduleMap = glob
  .sync('dist/**/*.js')
  .map(path => ({ key: path, value: findRequires(fs.readFileSync(path, 'utf8')) }));

const files = Object.fromEntries(
  moduleMap
    .map(({ key, value }) => ({
      key,
      value: value.filter(
        module =>
          !(deps.includes(module) || deps.some(dep => module.startsWith(`${dep}/`))),
      ),
    }))
    .filter(({ value }) => value.length > 0)
    .map(({ key, value }) => [key, value]),
);
const uselessDeps = Object.keys(dependencies).filter(
  dep => !moduleMap.some(({ value }) => value.includes(dep)),
);

console.log('Missing imports:', files);
console.log('Unused dependencies:', uselessDeps);
